/**
 * This module was automatically generated by `ts-interface-builder`
 */
import * as t from "ts-interface-checker";
export declare const Transform: t.TUnion;
export declare const SourceMapOptions: t.TIface;
export declare const Options: t.TIface;
declare const exportedTypeSuite: t.ITypeSuite;
export default exportedTypeSuite;
