export declare enum ContextualKeyword {
    NONE = 0,
    _abstract = 1,
    _as = 2,
    _asserts = 3,
    _async = 4,
    _await = 5,
    _checks = 6,
    _constructor = 7,
    _declare = 8,
    _enum = 9,
    _exports = 10,
    _from = 11,
    _get = 12,
    _global = 13,
    _implements = 14,
    _infer = 15,
    _interface = 16,
    _is = 17,
    _keyof = 18,
    _mixins = 19,
    _module = 20,
    _namespace = 21,
    _of = 22,
    _opaque = 23,
    _private = 24,
    _protected = 25,
    _proto = 26,
    _public = 27,
    _readonly = 28,
    _require = 29,
    _set = 30,
    _static = 31,
    _type = 32,
    _unique = 33
}
