import { Token } from "./parser/tokenizer";
export default class NameManager {
    private readonly usedNames;
    constructor(code: string, tokens: Array<Token>);
    claimFreeName(name: string): string;
    findFreeName(name: string): string;
}
