import { Options } from "./index";
export declare function addHook(extension: string, options: Options): void;
export declare function registerJS(): void;
export declare function registerJSX(): void;
export declare function registerTS(): void;
export declare function registerTSX(): void;
export declare function registerTSLegacyModuleInterop(): void;
export declare function registerTSXLegacyModuleInterop(): void;
export declare function registerAll(): void;
